@@ -100,6 +100,11 @@ void Location::setIdentifier(const QString &identifierType, const QString &id)
     d->ids.insert(identifierType, id);
 }
 
+bool Location::hasIdentifier(const QString &identifierType) const
+{
+    return !d->ids.value(identifierType).isEmpty();
+}
+
 RentalVehicleStation Location::rentalVehicleStation() const
 {
     return d->data.value<RentalVehicleStation>();
@@ -244,8 +249,9 @@ static int isSameDistanceThreshold(Location::Type type)
     switch (type) {
         case Location::Place:
         case Location::Stop:
+            return 25; // meter
         case Location::CarpoolPickupDropoff:
-            return 10; // meter
+            return 10;
         case Location::RentedVehicleStation:
             return 5;
         case Location::Equipment:
